import { ReactWidget } from '@jupyterlab/apputils';
import React from 'react';
const margin = {
    margin: '1em'
};
/**
 * React component for settings.
 *
 * @returns The React component
 * @param props Properties object that is typecasted to ISettingRegistry.ISettings
 */
const SettingsComponent = (props) => {
    const settings = props.settings;
    return (React.createElement("div", null,
        React.createElement("h1", { style: margin }, "EMBL-Tools Settngs"),
        React.createElement("h3", { style: margin, id: "currentEmail" },
            "Current email is: ",
            settings.get('email').composite),
        React.createElement("h3", { style: margin, id: "currentOutdir" },
            "Current output directtory is:",
            ' ',
            settings.get('outdir').composite),
        React.createElement("h2", { style: margin }, "Change Settings"),
        React.createElement("p", { style: margin },
            "Set default email: ",
            React.createElement("input", { type: "text", id: "email", name: "email" })),
        React.createElement("p", { style: margin },
            "Set default output directory:",
            React.createElement("input", { type: "text", id: "outdir", name: "outdir" })),
        React.createElement("br", null),
        React.createElement("br", null),
        React.createElement("button", { style: margin, onClick: async () => {
                if (document.getElementById('email').value ===
                    '' ||
                    document.getElementById('outdir').value === '') {
                    document.getElementById('warningText').innerHTML =
                        'Email and output directory cannot be empty.';
                    return;
                }
                if (!document.getElementById('email').value.match('.*@.*..*')) {
                    document.getElementById('warningText').innerHTML =
                        'Email does not seem to be valid. Please use the following format: username@provider.tld: eg.: someone@example.com';
                    return;
                }
                await settings
                    .set('email', document.getElementById('email').value)
                    .catch(reason => {
                    console.error(`Something went wrong when setting email.\n${reason}`);
                });
                let outdirClean = document.getElementById('outdir').value;
                if (outdirClean.endsWith('/')) {
                    outdirClean = outdirClean.substr(0, outdirClean.length - 1);
                }
                await settings.set('outdir', outdirClean).catch(reason => {
                    console.error(`Something went wrong when setting outdir.\n${reason}`);
                });
                document.getElementById('currentEmail').innerHTML = ('Current email is: ' +
                    settings.get('email').composite);
                document.getElementById('currentOutdir').innerHTML = ('Current output directtory is: ' +
                    settings.get('outdir').composite);
                document.getElementById('warningText').innerHTML = 'Settings saved.';
            } }, "Save settings"),
        React.createElement("br", null),
        React.createElement("h2", { id: "warningText", style: { color: 'red', margin: '1em' } })));
};
/**
 * The Lumio widget that wraps the settings ui React widget.
 */
export class SettingsWidget extends ReactWidget {
    /**
     * Constructs a new SettingsWidget.
     *
     * @param settings ISettingRegistry.ISettings object coming from the main extension when widget is created.
     */
    constructor(settings) {
        super();
        this.addClass('jp-ReactWidget');
        this.settings = settings;
    }
    render() {
        return React.createElement(SettingsComponent, { settings: this.settings });
    }
}
